class WebsocketServer
  Pairing = Struct.new(:is_ws, :proxy)

  def initialize
    @pairing = {}
    @sockets = Concurrent::Array.new

    Thread.new do
      loop do
        begin
          reads, writes, errors = IO.select(@sockets, @sockets, @sockets, 1)
        rescue IOError
          @sockets.select(&:closed?).each { |err| cleanup(err) }
        else
          Array(errors).each { |err| cleanup(err) }
        end

        # Skip this loop if we can't do anything
        if Array(reads).empty? || writes.empty? || errors.any?
          sleep(1) if @sockets.empty?
          next
        end

        # Do the data transfers
        reads.each do |socket|
          begin
            @pairing[socket].proxy.transmit(writes, @pairing[socket].is_ws)
          rescue
            cleanup(socket)
          end
        end
      end
    end
  end

  def call(env)
    if WebSocket::Driver.websocket?(env)
      exp = %r{^/ws/console/([a-zA-Z0-9]+)/?$}.match(env['REQUEST_URI'])
      return not_found if exp.nil?

      init_proxy(env, exp[1])

      [-1, {}, []]
    else
      not_found
    end
  end

  private

  def init_proxy(env, url)
    console = SystemConsole.find_by!(:url_secret => url)
    proxy = WebsocketProxy.new(env, console)
    return proxy.cleanup if proxy.error
    proxy.start

    # Release the connection because one SPICE console can open multiple TCP connections
    ActiveRecord::Base.connection_pool.release_connection

    # Get the descriptors and pass them to the worker thread
    ws, sock = proxy.descriptors
    @pairing.merge!(ws => Pairing.new(true, proxy), sock => Pairing.new(false, proxy))
    @sockets.push(ws, sock)
  end

  def cleanup(socket)
    @pairing[socket].proxy.cleanup
    @sockets.delete(socket)
    @pairing.delete(socket)
  end

  def not_found
    [404, {'Content-Type' => 'text/plain'}, ['Not found']]
  end
end
